/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QObject>
#include <QTest>
#include <QDebug>
#include <kolabobject.h>
#include "migrationutility/coordinationjob.h"
#include "migrationutility/kolabserver.h"
#include "migrationutility/sourceserver.h"
#include "lib/kolabaccount.h"
#include "lib/testlib/testutils.h"


/**
 * This test simulates an entier migration scenario.
 *
 * It requires two Kolab Servers, with all the required users already setup (ideally two local virtual machines, which are used for testing only)
 * So this is really a complex integration test, which involves all parts of the system, and not a unittest which has to be executed all the time.
 * On the other hand it should come quite close to a real world migration scenario and can also show problems with specific imap servers.
 *
 */
class MigrationScenarioTest: public QObject
{
    Q_OBJECT
    
    QString sourcehost;
    QString targethost;
    QString user;
    QString admin;
    QString adminpw;
    qint16 port;
    QList<Folder> folders;

    Object createEvent(const QString &uid)
    {
        Object calObj1;
        KCalCore::Event::Ptr event(new KCalCore::Event());
        event->setUid(uid);
        event->setDtStart(KDateTime::currentUtcDateTime());
        calObj1.object = QVariant::fromValue(Kolab::KolabObjectWriter::writeEvent(event, Kolab::KolabV2, "migrationtest"));
        calObj1.flags << QByteArray("\\Seen");
        return calObj1;

    }

public:
    explicit MigrationScenarioTest(QObject* parent = 0)
        : QObject(parent),
          sourcehost("192.168.122.104"),
          targethost("192.168.122.10"),
          user("john.doe@example.org"),
          admin("cyrus-admin"),
          adminpw("admin"),
          port(143)
    {
        folders << Folder("INBOX");
        folders << Folder("Calendar", Kolab::EventType, QList<Object>() << createEvent("uid1"));
        folders << Folder("Calendar/Personal Calendar", Kolab::EventType, QList<Object>() << createEvent("uid2"));
        folders << Folder("Configuration", Kolab::ConfigurationType);
        folders << Folder("Contacts", Kolab::ContactType);
        folders << Folder("Drafts");
        folders << Folder("Journal", Kolab::JournalType);
        folders << Folder("Notes", Kolab::NoteType);
        folders << Folder("Sent");
        folders << Folder("Tasks", Kolab::TaskType);
        folders << Folder("Trash");
    }

    virtual ~MigrationScenarioTest(){};

private:
    void setupSourceAccount()
    {
         KolabServer *kolabServer = new KolabServer(this);
         kolabServer->setHost(sourcehost, port);
         kolabServer->setAdminCredentials(admin, adminpw);
         KolabAccount *account = kolabServer->getAccount(user);
         account->cleanAccount();
         createFolders(account, folders);
    }

    void setupTargetAccount()
    {
        //Depending on the test scenario, clear target account
    }

    void executeMigration()
    {
        QObject obj;
        //Copy main.cpp
        KolabSourceServer *kolabSourceServer = new KolabSourceServer(&obj);
        kolabSourceServer->setHost(sourcehost, port);
        kolabSourceServer->setAdminCredentials(admin, adminpw);
        kolabSourceServer->setSingleUser(user);
        
        KolabServer *kolabServer = new KolabServer(&obj);
        kolabServer->setHost(targethost, port);
        kolabServer->setAdminCredentials(admin, adminpw);
        
        CoordinationJob *job = new CoordinationJob(kolabSourceServer, kolabServer, &obj);
        job->exec();
    }
    
    void checkFolders(SourceAccount *account, const QList<Folder> &folders)
    {
        const QStringList &receivedFolders = account->lookupFolderList();
        QStringList::const_iterator recIt = receivedFolders.constBegin();
        QList<Folder>::const_iterator foldersIt = folders.constBegin();
        QCOMPARE(receivedFolders.size(), folders.size());
        for (;foldersIt != folders.constEnd() && recIt != receivedFolders.constEnd(); ++foldersIt, ++recIt) {
            qDebug() << "Folder: " << *recIt;
            QCOMPARE(*recIt, foldersIt->name);
            //TODO Check folder annotations

            const QList<Object> &objects = account->getObjects(foldersIt->name);
            QCOMPARE(objects.size(), foldersIt->objects.size());
            
            QList<Object>::const_iterator recObjIt = objects.constBegin();
            QList<Object>::const_iterator objIt = foldersIt->objects.constBegin();
            for (;objIt != foldersIt->objects.constEnd() && recObjIt != objects.constEnd(); ++objIt, ++recObjIt) {
                QCOMPARE(*recObjIt->object.value<KCalCore::Incidence::Ptr>().dynamicCast<KCalCore::Event>(), *Kolab::KolabObjectReader(objIt->object.value<KMime::Message::Ptr>()).getEvent());
                foreach (const QByteArray &flag, objIt->flags) {
                    QVERIFY(recObjIt->flags.contains(flag));
                }
            }
        }
    }

    void checkTargetAccount()
    {
        
        QObject obj;
        KolabSourceServer *kolabSourceServer = new KolabSourceServer(&obj);
        kolabSourceServer->setHost(targethost, port);
        kolabSourceServer->setAdminCredentials(admin, adminpw);
        kolabSourceServer->setSingleUser(user);

        SourceAccount *account = kolabSourceServer->getSourceAccount(user);
        checkFolders(account, folders);
        account->logout();
    }

    void checkSourceAccount()
    {
        //If we start clearing the source server on successful migration, check that messages are really gone.
    }
    
private slots:
    void testMigration()
    {
        setupSourceAccount();
        setupTargetAccount();
        executeMigration();
        checkTargetAccount();
        checkSourceAccount();
    }

};
QTEST_MAIN( MigrationScenarioTest )

#include "migrationscenariotest.moc"